@@ -20,9 +20,6 @@
  * MA 02110-1301 USA.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #ifdef HAVE_XFCE_REVISION_H
 #include "xfce-revision.h"
 #endif
