@@ -43,10 +43,6 @@
  * stuff before 4.2!
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -67,20 +63,17 @@
 #include <X11/Xatom.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
-
 #include <gdk/gdkx.h>
-
+#include <libwnck/libwnck.h>
 #include <libxfce4ui/libxfce4ui.h>
 
-#include <libwnck/libwnck.h>
+#include "libxfsm/xfsm-util.h"
 
-#include <xfce4-session/xfsm-legacy.h>
-#include <libxfsm/xfsm-util.h>
+#include "xfsm-global.h"
+#include "xfsm-legacy.h"
 
 
-#ifdef LEGACY_SESSION_MANAGEMENT
-
-#define WM_SAVE_YOURSELF_TIMEOUT  4000
+#define WM_SAVE_YOURSELF_TIMEOUT 4000
 
 enum
 {
@@ -93,13 +86,13 @@ enum
 typedef struct _SmWindow SmWindow;
 struct _SmWindow
 {
-  gint    type;
+  gint type;
   gchar **wm_command;
-  gchar  *wm_client_machine;
-  gchar  *wm_class1;
-  gchar  *wm_class2;
-  Window  wid;
-  gint    screen_num;
+  gchar *wm_client_machine;
+  gchar *wm_class1;
+  gchar *wm_class2;
+  Window wid;
+  gint screen_num;
 };
 #define SM_WINDOW(window) ((SmWindow *) (window))
 
@@ -116,15 +109,18 @@ static GList *window_list = NULL;
 
 
 /* X Atoms */
-static Atom _XA_WM_PROTOCOLS     = None;
+static Atom _XA_WM_PROTOCOLS = None;
 static Atom _XA_WM_SAVE_YOURSELF = None;
 static Atom _XA_WM_CLIENT_LEADER = None;
-static Atom _XA_SM_CLIENT_ID     = None;
+static Atom _XA_SM_CLIENT_ID = None;
 
 
-static SmWindow*
-sm_window_new (Window wid, gint screen_num, gint type,
-               gchar *wm_class1, gchar *wm_class2)
+static SmWindow *
+sm_window_new (Window wid,
+               gint screen_num,
+               gint type,
+               gchar *wm_class1,
+               gchar *wm_class2)
 {
   SmWindow *smw;
 
@@ -199,7 +195,7 @@ has_xsmp_support (Window window)
   XTextProperty tp;
   gboolean has_it = FALSE;
 
-  if (XGetTextProperty (gdk_display, window, &tp, _XA_SM_CLIENT_ID))
+  if (XGetTextProperty (gdk_x11_get_default_xdisplay (), window, &tp, _XA_SM_CLIENT_ID))
     {
       if (tp.encoding == XA_STRING && tp.format == 8 && tp.nitems != 0)
         has_it = TRUE;
@@ -212,7 +208,7 @@ has_xsmp_support (Window window)
 }
 
 
-static gchar**
+static gchar **
 get_wmcommand (Window window)
 {
   gchar **result = NULL;
@@ -220,16 +216,19 @@ get_wmcommand (Window window)
   char **argv;
   int argc, i;
 
-  gdk_error_trap_push ();
-  status = XGetCommand (gdk_display, window, &argv, &argc);
-  if (gdk_error_trap_pop ())
+  gdk_x11_display_error_trap_push (gdk_display_get_default ());
+  status = XGetCommand (gdk_x11_get_default_xdisplay (), window, &argv, &argc);
+  if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()))
     return NULL;
 
   if (status && argv && argc > 0)
     {
       result = g_new0 (gchar *, argc + 1);
+
+      /* escape delimiter in command tokens */
       for (i = 0; i < argc; ++i)
-        result[i] = g_strdup (argv[i]);
+        result[i] = xfce_str_replace (argv[i], SESSION_FILE_DELIMITER, "\\" SESSION_FILE_DELIMITER);
+
       XFreeStringList (argv);
     }
 
@@ -237,16 +236,16 @@ get_wmcommand (Window window)
 }
 
 
-static gchar*
+static gchar *
 get_wmclientmachine (Window window)
 {
   XTextProperty tp;
   gchar *result = NULL;
   Status status;
 
-  gdk_error_trap_push ();
-  status = XGetWMClientMachine (gdk_display, window, &tp);
-  if (gdk_error_trap_pop ())
+  gdk_x11_display_error_trap_push (gdk_display_get_default ());
+  status = XGetWMClientMachine (gdk_x11_get_default_xdisplay (), window, &tp);
+  if (gdk_x11_display_error_trap_pop (gdk_display_get_default ()))
     return NULL;
 
   if (status)
@@ -274,26 +273,23 @@ get_clientleader (Window window)
   unsigned char *data = 0;
   Window result = window;
 
-  status = XGetWindowProperty (gdk_display, window, _XA_WM_CLIENT_LEADER,
+  status = XGetWindowProperty (gdk_x11_get_default_xdisplay (), window, _XA_WM_CLIENT_LEADER,
                                0, 10000, FALSE, XA_WINDOW, &type, &format,
                                &nitems, &extra, &data);
-  if (status  == Success)
+  if (status == Success)
     {
       if (data != NULL && nitems > 0)
-          result = *((Window *) data);
-      XFree(data);
+        result = *((Window *) (gpointer) data);
+      XFree (data);
     }
 
   return result;
 }
 
-#endif
-
 
 void
 xfsm_legacy_perform_session_save (void)
 {
-#ifdef LEGACY_SESSION_MANAGEMENT
   XErrorHandler old_handler;
   WnckScreen *screen;
   GList *windows;
@@ -309,7 +305,6 @@ xfsm_legacy_perform_session_save (void)
   gchar *wmclass1;
   gchar *wmclass2;
   Window root, window;
-  XEvent ev;
   int awaiting_replies = 0;
   GTimer *timer;
 
@@ -319,11 +314,11 @@ xfsm_legacy_perform_session_save (void)
   /* query X atoms */
   if (_XA_WM_SAVE_YOURSELF == None)
     {
-      _XA_SM_CLIENT_ID = XInternAtom (gdk_display, "SM_CLIENT_ID", False);
-      _XA_WM_PROTOCOLS = XInternAtom (gdk_display, "WM_PROTOCOLS", False);
-      _XA_WM_SAVE_YOURSELF = XInternAtom (gdk_display, "WM_SAVE_YOURSELF",
+      _XA_SM_CLIENT_ID = XInternAtom (gdk_x11_get_default_xdisplay (), "SM_CLIENT_ID", False);
+      _XA_WM_PROTOCOLS = XInternAtom (gdk_x11_get_default_xdisplay (), "WM_PROTOCOLS", False);
+      _XA_WM_SAVE_YOURSELF = XInternAtom (gdk_x11_get_default_xdisplay (), "WM_SAVE_YOURSELF",
                                           False);
-      _XA_WM_CLIENT_LEADER = XInternAtom (gdk_display, "WM_CLIENT_LEADER",
+      _XA_WM_CLIENT_LEADER = XInternAtom (gdk_x11_get_default_xdisplay (), "WM_CLIENT_LEADER",
                                           False);
     }
 
@@ -331,7 +326,7 @@ xfsm_legacy_perform_session_save (void)
   old_handler = XSetErrorHandler (wins_error_handler);
 
   /* query mapped windows on all screens */
-  for (n = 0; n < ScreenCount (gdk_display); ++n)
+  for (n = 0; n < ScreenCount (gdk_x11_get_default_xdisplay ()); ++n)
     {
       screen = wnck_screen_get (n);
       wnck_screen_force_update (screen);
@@ -344,7 +339,10 @@ xfsm_legacy_perform_session_save (void)
           leader = get_clientleader (window);
           if (leader == None || sm_window_list_contains (leader)
               || has_xsmp_support (window) || has_xsmp_support (leader))
-            continue;
+            {
+              xfsm_verbose ("window has no client leader or supports xspm, skipping\n");
+              continue;
+            }
 
           type = SM_WMCOMMAND;
           wmclass1 = NULL;
@@ -353,7 +351,7 @@ xfsm_legacy_perform_session_save (void)
           nprotocols = 0;
           protocols = NULL;
 
-          if (XGetWMProtocols (gdk_display, leader, &protocols, &nprotocols))
+          if (XGetWMProtocols (gdk_x11_get_default_xdisplay (), leader, &protocols, &nprotocols))
             {
               for (i = 0; i < nprotocols; ++i)
                 if (protocols[i] == _XA_WM_SAVE_YOURSELF)
@@ -364,7 +362,7 @@ xfsm_legacy_perform_session_save (void)
               XFree ((void *) protocols);
             }
 
-          if (XGetClassHint (gdk_display, leader, &class_hint))
+          if (XGetClassHint (gdk_x11_get_default_xdisplay (), leader, &class_hint))
             {
               wmclass2 = g_strdup (class_hint.res_class);
               wmclass1 = g_strdup (class_hint.res_name);
@@ -378,8 +376,8 @@ xfsm_legacy_perform_session_save (void)
     }
 
   /* open fresh display for sending WM_SAVE_YOURSELF commands */
-  XSync (gdk_display, False);
-  display = XOpenDisplay (DisplayString (gdk_display));
+  XSync (gdk_x11_get_default_xdisplay (), False);
+  display = XOpenDisplay (DisplayString (gdk_x11_get_default_xdisplay ()));
   if (display == NULL)
     {
       XSetErrorHandler (old_handler);
@@ -397,9 +395,9 @@ xfsm_legacy_perform_session_save (void)
       sm_window = SM_WINDOW (lp->data);
       if (sm_window->type == SM_WMSAVEYOURSELF)
         {
-          ++awaiting_replies;
+          XEvent ev = { 0 };
 
-          bzero (&ev, sizeof (ev));
+          ++awaiting_replies;
 
           ev.xclient.type = ClientMessage;
           ev.xclient.window = sm_window->wid;
@@ -420,9 +418,11 @@ xfsm_legacy_perform_session_save (void)
     {
       if (XPending (display))
         {
+          XEvent ev = { 0 };
+
           XNextEvent (display, &ev);
-          if (ev.type == UnmapNotify || (ev.type == PropertyNotify
-                && ev.xproperty.atom == XA_WM_COMMAND))
+          if (ev.type == UnmapNotify
+              || (ev.type == PropertyNotify && ev.xproperty.atom == XA_WM_COMMAND))
             {
               for (lp = window_list; lp != NULL; lp = lp->next)
                 {
@@ -440,19 +440,31 @@ xfsm_legacy_perform_session_save (void)
         {
           struct timeval tv;
           fd_set fds;
+          fd_set exceptfds;
           int msecs;
           int fd;
+          int ret;
 
-          msecs = (int)(g_timer_elapsed (timer, NULL) * 1000);
+          msecs = (int) (g_timer_elapsed (timer, NULL) * 1000);
           if (msecs >= WM_SAVE_YOURSELF_TIMEOUT)
             break;
 
           fd = ConnectionNumber (display);
           FD_ZERO (&fds);
           FD_SET (fd, &fds);
+          FD_ZERO (&exceptfds);
+          FD_SET (fd, &exceptfds);
           tv.tv_sec = (WM_SAVE_YOURSELF_TIMEOUT - msecs) / 1000;
           tv.tv_usec = ((WM_SAVE_YOURSELF_TIMEOUT - msecs) % 1000) * 1000;
-          select (fd + 1, &fds, NULL, &fds, &tv);
+          ret = select (fd + 1, &fds, NULL, &exceptfds, &tv);
+          if (ret == -1)
+            {
+              perror ("select");
+            }
+          else if (ret == 0)
+            {
+              g_warning ("xfsm_legacy_perform_session_save, select: no data before timeout reached");
+            }
         }
     }
   g_timer_destroy (timer);
@@ -475,14 +487,13 @@ xfsm_legacy_perform_session_save (void)
             sm_window->type = SM_ERROR;
         }
     }
-#endif
 }
 
 
 void
-xfsm_legacy_store_session (XfceRc *rc)
+xfsm_legacy_store_session (GKeyFile *file,
+                           const gchar *group)
 {
-#ifdef LEGACY_SESSION_MANAGEMENT
   int count = 0;
   SmWindow *sm_window;
   GList *lp;
@@ -495,37 +506,47 @@ xfsm_legacy_store_session (XfceRc *rc)
         {
           /* xmms is b0rked, surprise, surprise */
           if ((sm_window->wm_class1 != NULL
-                && strcmp (sm_window->wm_class1, "xmms") == 0)
+               && strcmp (sm_window->wm_class1, "xmms") == 0)
               || (sm_window->wm_class2 != NULL
-                && strcmp (sm_window->wm_class2, "xmms") == 0))
+                  && strcmp (sm_window->wm_class2, "xmms") == 0))
             continue;
 
           if (sm_window->wm_command == NULL
               || sm_window->wm_client_machine == NULL)
             continue;
 
+          if (xfsm_is_verbose_enabled ())
+            {
+              gchar *command = g_strjoinv (" ", sm_window->wm_command);
+              xfsm_verbose ("saving screen %d, command %s, machine %s\n",
+                            sm_window->screen_num,
+                            command,
+                            sm_window->wm_client_machine);
+              g_free (command);
+            }
+
           g_snprintf (buffer, 256, "Legacy%d_Screen", count);
-          xfce_rc_write_int_entry (rc, buffer, sm_window->screen_num);
+          g_key_file_set_integer (file, group, buffer, sm_window->screen_num);
 
           g_snprintf (buffer, 256, "Legacy%d_Command", count);
-          xfce_rc_write_list_entry (rc, buffer, sm_window->wm_command, NULL);
+          g_key_file_set_string_list (file, group, buffer, (const gchar *const *) sm_window->wm_command,
+                                      g_strv_length (sm_window->wm_command));
 
           g_snprintf (buffer, 256, "Legacy%d_ClientMachine", count);
-          xfce_rc_write_entry (rc, buffer, sm_window->wm_client_machine);
+          g_key_file_set_string (file, group, buffer, sm_window->wm_client_machine);
 
           ++count;
         }
     }
 
-  xfce_rc_write_int_entry (rc, "LegacyCount", count);
-#endif
+  g_key_file_set_integer (file, group, "LegacyCount", count);
 }
 
 
 void
-xfsm_legacy_load_session (XfceRc *rc)
+xfsm_legacy_load_session (GKeyFile *file,
+                          const gchar *group)
 {
-#ifdef LEGACY_SESSION_MANAGEMENT
   gchar buffer[256];
   int count;
   int i;
@@ -533,16 +554,36 @@ xfsm_legacy_load_session (XfceRc *rc)
   SmRestartApp *app;
   int screen_num;
 
-  count = xfce_rc_read_int_entry (rc, "LegacyCount", 0);
+  count = g_key_file_get_integer (file, group, "LegacyCount", NULL);
   for (i = 0; i < count; ++i)
     {
       g_snprintf (buffer, 256, "Legacy%d_Screen", i);
-      screen_num = xfce_rc_read_int_entry (rc, buffer, 0);
+      screen_num = g_key_file_get_integer (file, group, buffer, NULL);
 
       g_snprintf (buffer, 256, "Legacy%d_Command", i);
-      command = xfce_rc_read_list_entry (rc, buffer, NULL);
+      command = g_key_file_get_string_list (file, group, buffer, NULL, NULL);
       if (command == NULL)
-        continue;
+        {
+          xfsm_verbose ("legacy command == NULL\n");
+          continue;
+        }
+      else
+        {
+          /* unescape delimiter in command tokens */
+          for (gchar **token = command; *token != NULL; token++)
+            {
+              gchar *unesc_token = xfce_str_replace (*token, "\\" SESSION_FILE_DELIMITER, SESSION_FILE_DELIMITER);
+              g_free (*token);
+              *token = unesc_token;
+            }
+
+          if (xfsm_is_verbose_enabled ())
+            {
+              gchar *dbg_command = g_strjoinv (" ", command);
+              xfsm_verbose ("legacy command %s\n", dbg_command);
+              g_free (dbg_command);
+            }
+        }
 
       app = g_new0 (SmRestartApp, 1);
       app->screen_num = screen_num;
@@ -550,21 +591,19 @@ xfsm_legacy_load_session (XfceRc *rc)
 
       restart_apps = g_list_append (restart_apps, app);
     }
-#endif
 }
 
 
 void
 xfsm_legacy_init (void)
 {
-#ifdef LEGACY_SESSION_MANAGEMENT
   Atom dt_save_mode;
   Atom dt_restore_mode;
   Display *dpy;
   Window root;
   int n;
 
-  dpy = gdk_display;
+  dpy = gdk_x11_get_default_xdisplay ();
 
   /* Some CDE apps are broken (thanks again to Craig for the Sun Box :).
    * Bugfix found on http://bugzilla.gnome.org/long_list.cgi?buglist=81343
@@ -580,25 +619,22 @@ xfsm_legacy_init (void)
     {
       root = RootWindow (dpy, n);
       XChangeProperty (dpy, root, dt_save_mode, XA_STRING, 8,
-                       PropModeReplace, (unsigned char *)"xfce4", sizeof ("xfce4"));
+                       PropModeReplace, (unsigned char *) "xfce4", sizeof ("xfce4"));
       XChangeProperty (dpy, root, dt_restore_mode, XA_STRING, 8,
-                       PropModeReplace, (unsigned char *)"xfce4", sizeof ("xfce4"));
+                       PropModeReplace, (unsigned char *) "xfce4", sizeof ("xfce4"));
     }
-#endif
 }
 
 
 void
 xfsm_legacy_startup (void)
 {
-#ifdef LEGACY_SESSION_MANAGEMENT
   GdkScreen *screen;
   GList *lp;
 
   for (lp = restart_apps; lp != NULL; lp = lp->next)
     {
-      screen = gdk_display_get_screen (gdk_display_get_default (),
-                                       SM_RESTART_APP (lp->data)->screen_num);
+      screen = gdk_screen_get_default ();
       xfsm_start_application (SM_RESTART_APP (lp->data)->command, NULL,
                               screen, NULL, NULL, NULL);
       g_strfreev (SM_RESTART_APP (lp->data)->command);
@@ -607,34 +643,28 @@ xfsm_legacy_startup (void)
 
   g_list_free (restart_apps);
   restart_apps = NULL;
-#endif
 }
 
 
 void
 xfsm_legacy_shutdown (void)
 {
-#ifdef LEGACY_SESSION_MANAGEMENT
   SmWindow *sm_window;
   GList *lp;
 
-  gdk_error_trap_push ();
+  gdk_x11_display_error_trap_push (gdk_display_get_default ());
 
   /* kill 'em all! */
   for (lp = window_list; lp != NULL; lp = lp->next)
     {
       sm_window = SM_WINDOW (lp->data);
       if (sm_window->wid != None)
-        XKillClient (gdk_display, sm_window->wid);
+        XKillClient (gdk_x11_get_default_xdisplay (), sm_window->wid);
     }
 
   sm_window_list_clear ();
 
-  gdk_flush ();
+  gdk_display_flush (gdk_display_get_default ());
 
-  gdk_error_trap_pop ();
-#endif
+  gdk_x11_display_error_trap_pop_ignored (gdk_display_get_default ());
 }
-
-
-
